# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations


class RespeecherEnvironment:
    PUBLIC_EN_RT: RespeecherEnvironment

    def __init__(self, *, base: str, ws: str):
        self.base = base
        self.ws = ws


RespeecherEnvironment.PUBLIC_EN_RT = RespeecherEnvironment(
    base="https://api.respeecher.com/v1/public/tts/en-rt", ws="wss://api.respeecher.com/v1/public/tts/en-rt"
)



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations

import typing

import typing_extensions


class ServerSentEvent_ChunkParams(typing_extensions.TypedDict):
    type: typing.Literal["chunk"]
    data: str


class ServerSentEvent_ErrorParams(typing_extensions.TypedDict):
    type: typing.Literal["error"]
    error: str
    status_code: int
    context_id: typing_extensions.NotRequired[str]


ServerSentEventParams = typing.Union[ServerSentEvent_ChunkParams, ServerSentEvent_ErrorParams]



# This file was auto-generated by Fern from our API Definition.

import typing_extensions


class StreamingErrorParams(typing_extensions.TypedDict):
    error: str
    """
    Error message.
    """

    status_code: int
    """
    HTTP status code most appropriate for this error.
    """

    context_id: typing_extensions.NotRequired[str]



# This file was auto-generated by Fern from our API Definition.

# isort: skip_file

from .sampling_params import SamplingParamsParams
from .voice import VoiceParams

__all__ = ["SamplingParamsParams", "VoiceParams"]



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations

import typing

import typing_extensions


class Response_ChunkParams(typing_extensions.TypedDict):
    type: typing.Literal["chunk"]
    context_id: str
    data: str


class Response_DoneParams(typing_extensions.TypedDict):
    type: typing.Literal["done"]
    context_id: str


class Response_ErrorParams(typing_extensions.TypedDict):
    type: typing.Literal["error"]
    error: str
    status_code: int
    context_id: typing_extensions.NotRequired[str]


ResponseParams = typing.Union[Response_ChunkParams, Response_DoneParams, Response_ErrorParams]



# This file was auto-generated by Fern from our API Definition.

import typing
from contextlib import asynccontextmanager, contextmanager

import websockets
import websockets.sync.client as websockets_sync_client
from ..core.api_error import ApiError
from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.request_options import RequestOptions
from .raw_client import AsyncRawTtsClient, RawTtsClient
from .requests.output_format import OutputFormatParams
from .requests.streaming_output_format import StreamingOutputFormatParams
from .requests.voice import VoiceParams
from .socket_client import AsyncTtsSocketClient, TtsSocketClient
from .types.server_sent_event import ServerSentEvent

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class TtsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawTtsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawTtsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawTtsClient
        """
        return self._raw_client

    def bytes(
        self,
        *,
        transcript: str,
        voice: VoiceParams,
        output_format: typing.Optional[OutputFormatParams] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> typing.Iterator[bytes]:
        """
        The easiest way to generate text-to-speech audio. Not suitable for latency-sensitive applications.

        Parameters
        ----------
        transcript : str
            Text for narration.

        voice : VoiceParams
            Voice for narration.

        output_format : typing.Optional[OutputFormatParams]
            Audio format specification.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration. You can pass in configuration such as `chunk_size`, and more to customize the request and response.

        Returns
        -------
        typing.Iterator[bytes]
            WAV file (16-bit LE PCM).

        Examples
        --------
        from respeecher import Respeecher

        client = Respeecher(
            api_key="YOUR_API_KEY",
        )
        client.tts.bytes(
            transcript="Hello, World!",
            voice={"id": "samantha"},
        )
        """
        with self._raw_client.bytes(
            transcript=transcript, voice=voice, output_format=output_format, request_options=request_options
        ) as r:
            yield from r.data

    def sse(
        self,
        *,
        transcript: str,
        voice: VoiceParams,
        output_format: typing.Optional[StreamingOutputFormatParams] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> typing.Iterator[ServerSentEvent]:
        """
        Stream text-to-speech audio as JSONL (JSON lines) objects over HTTP. A less performant alternative to WebSockets, without text input streaming.

        Parameters
        ----------
        transcript : str
            Text for narration.

        voice : VoiceParams
            Voice for narration.

        output_format : typing.Optional[StreamingOutputFormatParams]
            Audio format specification.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Yields
        ------
        typing.Iterator[ServerSentEvent]

        Examples
        --------
        from respeecher import Respeecher

        client = Respeecher(
            api_key="YOUR_API_KEY",
        )
        response = client.tts.sse(
            transcript="Hello, World!",
            voice={"id": "samantha"},
        )
        for chunk in response:
            yield chunk
        """
        with self._raw_client.sse(
            transcript=transcript, voice=voice, output_format=output_format, request_options=request_options
        ) as r:
            yield from r.data

    @contextmanager
    def connect(self, *, request_options: typing.Optional[RequestOptions] = None) -> typing.Iterator[TtsSocketClient]:
        """
        A single connection for multiple concurrent text-to-speech generations, with input and output streaming. Provides the best latency and performance out of the text-to-speech endpoints.

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        TtsSocketClient
        """
        ws_url = self._raw_client._client_wrapper.get_environment().ws + "/tts/websocket"
        headers = self._raw_client._client_wrapper.get_headers()
        if request_options and "additional_headers" in request_options:
            headers.update(request_options["additional_headers"])
        try:
            with websockets_sync_client.connect(ws_url, additional_headers=headers) as protocol:
                yield TtsSocketClient(websocket=protocol)
        except websockets.exceptions.InvalidStatusCode as exc:
            status_code: int = exc.status_code
            if status_code == 401:
                raise ApiError(
                    status_code=status_code,
                    headers=dict(headers),
                    body="Websocket initialized with invalid credentials.",
                )
            raise ApiError(
                status_code=status_code,
                headers=dict(headers),
                body="Unexpected error when initializing websocket connection.",
            )


class AsyncTtsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawTtsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawTtsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawTtsClient
        """
        return self._raw_client

    async def bytes(
        self,
        *,
        transcript: str,
        voice: VoiceParams,
        output_format: typing.Optional[OutputFormatParams] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> typing.AsyncIterator[bytes]:
        """
        The easiest way to generate text-to-speech audio. Not suitable for latency-sensitive applications.

        Parameters
        ----------
        transcript : str
            Text for narration.

        voice : VoiceParams
            Voice for narration.

        output_format : typing.Optional[OutputFormatParams]
            Audio format specification.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration. You can pass in configuration such as `chunk_size`, and more to customize the request and response.

        Returns
        -------
        typing.AsyncIterator[bytes]
            WAV file (16-bit LE PCM).

        Examples
        --------
        import asyncio

        from respeecher import AsyncRespeecher

        client = AsyncRespeecher(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.tts.bytes(
                transcript="Hello, World!",
                voice={"id": "samantha"},
            )


        asyncio.run(main())
        """
        async with self._raw_client.bytes(
            transcript=transcript, voice=voice, output_format=output_format, request_options=request_options
        ) as r:
            async for _chunk in r.data:
                yield _chunk

    async def sse(
        self,
        *,
        transcript: str,
        voice: VoiceParams,
        output_format: typing.Optional[StreamingOutputFormatParams] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> typing.AsyncIterator[ServerSentEvent]:
        """
        Stream text-to-speech audio as JSONL (JSON lines) objects over HTTP. A less performant alternative to WebSockets, without text input streaming.

        Parameters
        ----------
        transcript : str
            Text for narration.

        voice : VoiceParams
            Voice for narration.

        output_format : typing.Optional[StreamingOutputFormatParams]
            Audio format specification.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Yields
        ------
        typing.AsyncIterator[ServerSentEvent]

        Examples
        --------
        import asyncio

        from respeecher import AsyncRespeecher

        client = AsyncRespeecher(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            response = await client.tts.sse(
                transcript="Hello, World!",
                voice={"id": "samantha"},
            )
            async for chunk in response:
                yield chunk


        asyncio.run(main())
        """
        async with self._raw_client.sse(
            transcript=transcript, voice=voice, output_format=output_format, request_options=request_options
        ) as r:
            async for _chunk in r.data:
                yield _chunk

    @asynccontextmanager
    async def connect(
        self, *, request_options: typing.Optional[RequestOptions] = None
    ) -> typing.AsyncIterator[AsyncTtsSocketClient]:
        """
        A single connection for multiple concurrent text-to-speech generations, with input and output streaming. Provides the best latency and performance out of the text-to-speech endpoints.

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncTtsSocketClient
        """
        ws_url = self._raw_client._client_wrapper.get_environment().ws + "/tts/websocket"
        headers = self._raw_client._client_wrapper.get_headers()
        if request_options and "additional_headers" in request_options:
            headers.update(request_options["additional_headers"])
        try:
            async with websockets.connect(ws_url, extra_headers=headers) as protocol:
                yield AsyncTtsSocketClient(websocket=protocol)
        except websockets.exceptions.InvalidStatusCode as exc:
            status_code: int = exc.status_code
            if status_code == 401:
                raise ApiError(
                    status_code=status_code,
                    headers=dict(headers),
                    body="Websocket initialized with invalid credentials.",
                )
            raise ApiError(
                status_code=status_code,
                headers=dict(headers),
                body="Unexpected error when initializing websocket connection.",
            )



# This file was auto-generated by Fern from our API Definition.

import typing_extensions


class OutputFormatParams(typing_extensions.TypedDict):
    sample_rate: typing_extensions.NotRequired[int]
    """
    Audio sample rate, defaults to 22050.
    """



# This file was auto-generated by Fern from our API Definition.

# isort: skip_file

from .bytes_generation_request import BytesGenerationRequest
from .cancellation_request import CancellationRequest
from .chunk import Chunk
from .contextful_chunk import ContextfulChunk
from .contextful_generation_request import ContextfulGenerationRequest
from .done import Done
from .generation_request import GenerationRequest
from .output_format import OutputFormat
from .response import Response, Response_Chunk, Response_Done, Response_Error
from .server_sent_event import ServerSentEvent, ServerSentEvent_Chunk, ServerSentEvent_Error
from .streaming_encoding import StreamingEncoding
from .streaming_error import StreamingError
from .streaming_generation_request import StreamingGenerationRequest
from .streaming_output_format import StreamingOutputFormat
from .voice import Voice

__all__ = [
    "BytesGenerationRequest",
    "CancellationRequest",
    "Chunk",
    "ContextfulChunk",
    "ContextfulGenerationRequest",
    "Done",
    "GenerationRequest",
    "OutputFormat",
    "Response",
    "Response_Chunk",
    "Response_Done",
    "Response_Error",
    "ServerSentEvent",
    "ServerSentEvent_Chunk",
    "ServerSentEvent_Error",
    "StreamingEncoding",
    "StreamingError",
    "StreamingGenerationRequest",
    "StreamingOutputFormat",
    "Voice",
]



# This file was auto-generated by Fern from our API Definition.

# isort: skip_file

from .types import Gender, SamplingParams, Voice
from .requests import SamplingParamsParams, VoiceParams

__all__ = ["Gender", "SamplingParams", "SamplingParamsParams", "Voice", "VoiceParams"]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ...core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel


class SamplingParams(UniversalBaseModel):
    seed: typing.Optional[int] = pydantic.Field(default=None)
    """
    Generations with the same parameters _including_ a non-null `seed` are identical.
    """

    temperature: typing.Optional[float] = pydantic.Field(default=None)
    """
    Smaller values correspond to more stable but less expressive speech. Must be greater than or equal to 0.
    """

    top_k: typing.Optional[int] = pydantic.Field(default=None)
    """
    Must be -1 or greater than 0.
    """

    top_p: typing.Optional[float] = pydantic.Field(default=None)
    """
    Must be greater than 0 and less than or equal to 1.
    """

    min_p: typing.Optional[float] = pydantic.Field(default=None)
    """
    Must be between 0 and 1, inclusive.
    """

    repetition_penalty: typing.Optional[float] = pydantic.Field(default=None)
    """
    Must be between 1 and 2, inclusive.
    """

    frequency_penalty: typing.Optional[float] = pydantic.Field(default=None)
    """
    Must be between 0 and 2, inclusive.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

# isort: skip_file

from .gender import Gender
from .sampling_params import SamplingParams
from .voice import Voice

__all__ = ["Gender", "SamplingParams", "Voice"]



# This file was auto-generated by Fern from our API Definition.

import json
import typing

import websockets
import websockets.sync.connection as websockets_sync_connection
from ..core.events import EventEmitterMixin, EventType
from ..core.pydantic_utilities import parse_obj_as
from .types.cancellation_request import CancellationRequest
from .types.contextful_generation_request import ContextfulGenerationRequest
from .types.response import Response

TtsSocketClientResponse = typing.Union[Response]


class AsyncTtsSocketClient(EventEmitterMixin):
    def __init__(self, *, websocket: websockets.WebSocketClientProtocol):
        super().__init__()
        self._websocket = websocket

    async def __aiter__(self):
        async for message in self._websocket:
            yield parse_obj_as(TtsSocketClientResponse, message)  # type: ignore

    async def start_listening(self):
        """
        Start listening for messages on the websocket connection.

        Emits events in the following order:
        - EventType.OPEN when connection is established
        - EventType.MESSAGE for each message received
        - EventType.ERROR if an error occurs
        - EventType.CLOSE when connection is closed
        """
        self._emit(EventType.OPEN, None)
        try:
            async for raw_message in self._websocket:
                parsed = parse_obj_as(TtsSocketClientResponse, raw_message)  # type: ignore
                self._emit(EventType.MESSAGE, parsed)
        except websockets.WebSocketException as exc:
            self._emit(EventType.ERROR, exc)
        finally:
            self._emit(EventType.CLOSE, None)

    async def send_generate(self, message: ContextfulGenerationRequest) -> None:
        """
        Send a message to the websocket connection.
        The message will be sent as a ContextfulGenerationRequest.
        """
        await self._send_model(message)

    async def send_cancel(self, message: CancellationRequest) -> None:
        """
        Send a message to the websocket connection.
        The message will be sent as a CancellationRequest.
        """
        await self._send_model(message)

    async def recv(self) -> TtsSocketClientResponse:
        """
        Receive a message from the websocket connection.
        """
        data = await self._websocket.recv()
        return parse_obj_as(TtsSocketClientResponse, data)  # type: ignore

    async def _send(self, data: typing.Any) -> None:
        """
        Send a message to the websocket connection.
        """
        if isinstance(data, dict):
            data = json.dumps(data)
        await self._websocket.send(data)

    async def _send_model(self, data: typing.Any) -> None:
        """
        Send a Pydantic model to the websocket connection.
        """
        await self._send(data.dict())


class TtsSocketClient(EventEmitterMixin):
    def __init__(self, *, websocket: websockets_sync_connection.Connection):
        super().__init__()
        self._websocket = websocket

    def __iter__(self):
        for message in self._websocket:
            yield parse_obj_as(TtsSocketClientResponse, message)  # type: ignore

    def start_listening(self):
        """
        Start listening for messages on the websocket connection.

        Emits events in the following order:
        - EventType.OPEN when connection is established
        - EventType.MESSAGE for each message received
        - EventType.ERROR if an error occurs
        - EventType.CLOSE when connection is closed
        """
        self._emit(EventType.OPEN, None)
        try:
            for raw_message in self._websocket:
                parsed = parse_obj_as(TtsSocketClientResponse, raw_message)  # type: ignore
                self._emit(EventType.MESSAGE, parsed)
        except websockets.WebSocketException as exc:
            self._emit(EventType.ERROR, exc)
        finally:
            self._emit(EventType.CLOSE, None)

    def send_generate(self, message: ContextfulGenerationRequest) -> None:
        """
        Send a message to the websocket connection.
        The message will be sent as a ContextfulGenerationRequest.
        """
        self._send_model(message)

    def send_cancel(self, message: CancellationRequest) -> None:
        """
        Send a message to the websocket connection.
        The message will be sent as a CancellationRequest.
        """
        self._send_model(message)

    def recv(self) -> TtsSocketClientResponse:
        """
        Receive a message from the websocket connection.
        """
        data = self._websocket.recv()
        return parse_obj_as(TtsSocketClientResponse, data)  # type: ignore

    def _send(self, data: typing.Any) -> None:
        """
        Send a message to the websocket connection.
        """
        if isinstance(data, dict):
            data = json.dumps(data)
        self._websocket.send(data)

    def _send_model(self, data: typing.Any) -> None:
        """
        Send a Pydantic model to the websocket connection.
        """
        self._send(data.dict())



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
from ...core.serialization import FieldMetadata
from .streaming_generation_request import StreamingGenerationRequestParams


class ContextfulGenerationRequestParams(StreamingGenerationRequestParams):
    context_id: str
    """
    Use different context IDs for different generations over one WebSocket connection.
    """

    continue_: typing_extensions.NotRequired[typing_extensions.Annotated[bool, FieldMetadata(alias="continue")]]
    """
    Set to `true` for smooth prosody if text is streamed in chunks and this is not the last chunk.
    """



# This file was auto-generated by Fern from our API Definition.

import typing
from enum import Enum


class EventType(str, Enum):
    OPEN = "open"
    MESSAGE = "message"
    ERROR = "error"
    CLOSE = "close"


class EventEmitterMixin:
    """
    Simple mixin for registering and emitting events.
    """

    def __init__(self) -> None:
        self._callbacks: typing.Dict[EventType, typing.List[typing.Callable]] = {}

    def on(self, event_name: EventType, callback: typing.Callable[[typing.Any], None]) -> None:
        if event_name not in self._callbacks:
            self._callbacks[event_name] = []
        self._callbacks[event_name].append(callback)

    def _emit(self, event_name: EventType, data: typing.Any) -> None:
        if event_name in self._callbacks:
            for cb in self._callbacks[event_name]:
                cb(data)



# This file was auto-generated by Fern from our API Definition.

import typing

import httpx
from ..environment import RespeecherEnvironment
from .http_client import AsyncHttpClient, HttpClient


class BaseClientWrapper:
    def __init__(self, *, api_key: str, environment: RespeecherEnvironment, timeout: typing.Optional[float] = None):
        self.api_key = api_key
        self._environment = environment
        self._timeout = timeout

    def get_headers(self) -> typing.Dict[str, str]:
        headers: typing.Dict[str, str] = {
            "User-Agent": "respeecher/0.1.0",
            "X-Fern-Language": "Python",
            "X-Fern-SDK-Name": "respeecher",
            "X-Fern-SDK-Version": "0.1.0",
        }
        headers["X-API-Key"] = self.api_key
        return headers

    def get_environment(self) -> RespeecherEnvironment:
        return self._environment

    def get_timeout(self) -> typing.Optional[float]:
        return self._timeout


class SyncClientWrapper(BaseClientWrapper):
    def __init__(
        self,
        *,
        api_key: str,
        environment: RespeecherEnvironment,
        timeout: typing.Optional[float] = None,
        httpx_client: httpx.Client,
    ):
        super().__init__(api_key=api_key, environment=environment, timeout=timeout)
        self.httpx_client = HttpClient(
            httpx_client=httpx_client, base_headers=self.get_headers, base_timeout=self.get_timeout
        )


class AsyncClientWrapper(BaseClientWrapper):
    def __init__(
        self,
        *,
        api_key: str,
        environment: RespeecherEnvironment,
        timeout: typing.Optional[float] = None,
        httpx_client: httpx.AsyncClient,
    ):
        super().__init__(api_key=api_key, environment=environment, timeout=timeout)
        self.httpx_client = AsyncHttpClient(
            httpx_client=httpx_client, base_headers=self.get_headers, base_timeout=self.get_timeout
        )



# This file was auto-generated by Fern from our API Definition.

# isort: skip_file

from . import tts, voices
from .client import AsyncRespeecher, Respeecher
from .environment import RespeecherEnvironment
from .version import __version__
from .voices import Gender, SamplingParams, SamplingParamsParams, Voice, VoiceParams

__all__ = [
    "AsyncRespeecher",
    "Gender",
    "Respeecher",
    "RespeecherEnvironment",
    "SamplingParams",
    "SamplingParamsParams",
    "Voice",
    "VoiceParams",
    "__version__",
    "tts",
    "voices",
]



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
from ..types.gender import Gender
from .sampling_params import SamplingParamsParams


class VoiceParams(typing_extensions.TypedDict):
    id: str
    gender: typing_extensions.NotRequired[Gender]
    accent: typing_extensions.NotRequired[str]
    age: typing_extensions.NotRequired[str]
    sampling_params: typing_extensions.NotRequired[SamplingParamsParams]
    """
    Default sampling params for this voice.
    """



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
from ..types.streaming_encoding import StreamingEncoding
from .output_format import OutputFormatParams


class StreamingOutputFormatParams(OutputFormatParams):
    encoding: typing_extensions.NotRequired[StreamingEncoding]



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
from .generation_request import GenerationRequestParams
from .output_format import OutputFormatParams


class BytesGenerationRequestParams(GenerationRequestParams):
    output_format: typing_extensions.NotRequired[OutputFormatParams]
    """
    Audio format specification.
    """
